package javax.net.ssl;

import java.util.regex.Pattern;
import java.util.Locale;
import java.nio.charset.CharacterCodingException;
import java.nio.ByteBuffer;
import java.nio.charset.CodingErrorAction;
import java.nio.charset.StandardCharsets;
import java.net.IDN;
import java.util.Objects;

public final class SNIHostName extends SNIServerName
{
    private final String hostname;
    
    public SNIHostName(String ascii) {
        super(0, (ascii = IDN.toASCII((String)Objects.requireNonNull(ascii, "Server name value of host_name cannot be null"), 2)).getBytes(StandardCharsets.US_ASCII));
        this.hostname = ascii;
        this.checkHostName();
    }
    
    public SNIHostName(byte[] array) {
        super(0, array);
        try {
            this.hostname = IDN.toASCII(StandardCharsets.UTF_8.newDecoder().onMalformedInput(CodingErrorAction.REPORT).onUnmappableCharacter(CodingErrorAction.REPORT).decode(ByteBuffer.wrap(array)).toString());
        }
        catch (RuntimeException | CharacterCodingException cause) {
            throw new IllegalArgumentException("The encoded server name value is invalid", cause);
        }
        this.checkHostName();
    }
    
    public String getAsciiName() {
        return this.hostname;
    }
    
    @Override
    public boolean equals(Object o) {
        return this == o || (o instanceof SNIHostName && this.hostname.equalsIgnoreCase(((SNIHostName)o).hostname));
    }
    
    @Override
    public int hashCode() {
        return 31 * 17 + this.hostname.toUpperCase(Locale.ENGLISH).hashCode();
    }
    
    @Override
    public String toString() {
        return "type=host_name (0), value=" + this.hostname;
    }
    
    public static SNIMatcher createSNIMatcher(String s) {
        if (s == null) {
            throw new NullPointerException("The regular expression cannot be null");
        }
        return new SNIHostNameMatcher(s);
    }
    
    private void checkHostName() {
        if (this.hostname.isEmpty()) {
            throw new IllegalArgumentException("Server name value of host_name cannot be empty");
        }
        if (this.hostname.endsWith(".")) {
            throw new IllegalArgumentException("Server name value of host_name cannot have the trailing dot");
        }
        return;
    }
    
    private static final class SNIHostNameMatcher extends SNIMatcher
    {
        private final Pattern pattern;
        
        SNIHostNameMatcher(String regex) {
            super(0);
            this.pattern = Pattern.compile(regex, 2);
        }
        
        @Override
        public boolean matches(SNIServerName sniServerName) {
            if (sniServerName == null) {
                throw new NullPointerException("The SNIServerName argument cannot be null");
            }
            SNIHostName sniHostName;
            if (!(sniServerName instanceof SNIHostName)) {
                if (sniServerName.getType() != 0) {
                    throw new IllegalArgumentException("The server name type is not host_name");
                }
                try {
                    sniHostName = new SNIHostName(sniServerName.getEncoded());
                }
                catch (NullPointerException | IllegalArgumentException ex) {
                    return false;
                }
            }
            else {
                sniHostName = (SNIHostName)sniServerName;
            }
            String asciiName = sniHostName.getAsciiName();
            return this.pattern.matcher(asciiName).matches() || this.pattern.matcher(IDN.toUnicode(asciiName)).matches();
        }
    }
}
